@@ -1,5 +1,5 @@
 /*
-  A candidate replacement for Pspp's sheet
+  A widget to display and manipulate tabular data
   Copyright (C) 2016, 2017  John Darrington
 
   This program is free software: you can redistribute it and/or modify
@@ -41,7 +41,6 @@
 
 #include <config.h>
 #include "ssw-sheet-axis.h"
-#include "ssw-datum.h"
 #include "ssw-marshaller.h"
 #include <math.h>
 #include <stdlib.h>
@@ -305,17 +304,15 @@ button_released (GtkGesture *g,
 static GtkWidget *
 get_widget (SswSheetAxis *axis, guint index)
 {
-  gpointer item;
-  GtkWidget *old_widget = NULL;
-  GtkWidget *new_widget;
   PRIV_DECL (axis);
 
-  item = g_list_model_get_item (priv->model, index);
+  GtkWidget *new_widget = g_list_model_get_item (priv->model, index);
 
   if (priv->pool->len > 0)
-    old_widget = g_ptr_array_remove_index_fast (priv->pool, 0);
-
-  new_widget = cell_fill_func (axis, item, old_widget, index);
+    {
+      GtkWidget *old_widget = g_ptr_array_remove_index_fast (priv->pool, 0);
+      g_object_unref (old_widget);
+    }
 
   g_object_set_data (G_OBJECT (new_widget), "item-id", GINT_TO_POINTER (index));
 
@@ -824,7 +821,8 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
 static void
 value_changed_cb (GtkAdjustment *adjustment, gpointer user_data)
 {
-  ensure_visible_widgets (user_data, FALSE);
+  SswSheetAxis *axis = SSW_SHEET_AXIS (user_data);
+  ensure_visible_widgets (axis, FALSE);
 }
 
 static void
@@ -832,12 +830,15 @@ items_changed_cb (GListModel *model,
                   guint position,
                   guint removed, guint added, gpointer user_data)
 {
-  SswSheetAxis *axis = user_data;
+  SswSheetAxis *axis = SSW_SHEET_AXIS (user_data);
   gint i;
-  PRIV_DECL (user_data);
+  PRIV_DECL (axis);
 
+  if (! gtk_widget_get_realized (GTK_WIDGET (axis)))
+    return;
+  
   /* If the change is out of our visible range anyway,
-   *we don't care. */
+     then we don't care. */
   if (position > priv->model_to && bin_window_full (axis))
     {
       configure_adjustment (axis);
@@ -1125,6 +1126,8 @@ __finalize (GObject *obj)
   g_hash_table_destroy (priv->size_override);
   g_ptr_array_free (priv->pool, TRUE);
   g_ptr_array_free (priv->widgets, TRUE);
+  gtk_target_list_unref (priv->drag_target_list);
+
 
   if (axis->cell_limits)
     g_ptr_array_free (axis->cell_limits, TRUE);
@@ -1159,22 +1162,18 @@ ssw_sheet_axis_set_model (SswSheetAxis *axis, GListModel *model)
   PRIV_DECL (axis);
 
   if (priv->model != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (priv->model,
-                                            G_CALLBACK (items_changed_cb),
-                                            axis);
-      g_object_unref (priv->model);
-    }
+    g_object_unref (priv->model);
 
   PRIV (axis)->model = model;
   if (model != NULL)
     {
-      g_signal_connect (G_OBJECT (model), "items-changed",
-                        G_CALLBACK (items_changed_cb), axis);
+      g_signal_connect_object (G_OBJECT (model), "items-changed",
+			       G_CALLBACK (items_changed_cb), axis, 0);
       g_object_ref (model);
     }
 
-  g_signal_connect_swapped (model, "notify", G_CALLBACK (force_update), axis);
+  g_signal_connect_object (model, "notify", G_CALLBACK (force_update),
+			   axis, G_CONNECT_SWAPPED);
   ensure_visible_widgets (axis, TRUE);
 }
 
@@ -1897,7 +1896,7 @@ void
 ssw_sheet_axis_jump_end_with_offset (SswSheetAxis *axis, gint whereto, gint offs)
 {
   PRIV_DECL (axis);
-  g_return_if_fail (whereto < ssw_sheet_axis_get_size (axis));
+  g_return_if_fail (whereto <= ssw_sheet_axis_get_size (axis));
 
   gdouble upper = gtk_adjustment_get_upper (priv->adjustment);
   gdouble page_size = gtk_adjustment_get_page_size (priv->adjustment);
