@@ -1,5 +1,5 @@
 /*
-  A candidate replacement for Pspp's sheet
+  A widget to display and manipulate tabular data
   Copyright (C) 2016, 2017  John Darrington
 
   This program is free software: you can redistribute it and/or modify
@@ -146,6 +146,9 @@ get_active_cell (SswSheetBody *body, gint *col, gint *row)
   *col = priv->selection->start_x;
   *row = priv->selection->start_y;
 
+  if (*col < 0 || *row < 0)
+    return FALSE;
+
   return TRUE;
 }
 
@@ -233,23 +236,41 @@ draw_selection (SswSheetBody *body, cairo_t *cr)
   guint height = gtk_widget_get_allocated_height (w);
   PRIV_DECL (body);
 
+  GdkRGBA *color;
+  GtkStyleContext *sc = gtk_widget_get_style_context (w);
+  gtk_style_context_get (sc,
+			 GTK_STATE_FLAG_SELECTED,
+			 "background-color",
+			 &color, NULL);
+
+  GtkCssProvider *cp = gtk_css_provider_new ();
+  gchar *css = g_strdup_printf ("* {background-color: rgba(%d, %d, %d, 0.25);}",
+				(int) (100.0 * color->red),
+				(int) (100.0 * color->green),
+				(int) (100.0 * color->blue));
+
+  gtk_css_provider_load_from_data (cp, css, strlen (css), 0);
+
+  gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (cp),
+				  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
   /* Draw the selection */
   if ((priv->selection->start_x != priv->selection->end_x)
       ||
       (priv->selection->start_y != priv->selection->end_y))
     {
-      GdkRGBA color;
+
       SswRange mySelect;
       normalise_selection (priv->selection, &mySelect);
 
       gint xpos_start, ypos_start;
       if (0 != ssw_sheet_axis_find_boundary (priv->haxis, mySelect.start_x,
 					     &xpos_start, NULL))
-	return;
+	goto done;
 
       if (0 != ssw_sheet_axis_find_boundary (priv->vaxis, mySelect.start_y,
 					     &ypos_start, NULL))
-	return;
+	goto done;
 
       gint xlimit = width;
       gint ylimit = height;
@@ -263,33 +284,18 @@ draw_selection (SswSheetBody *body, cairo_t *cr)
 					     mySelect.end_y, &ypos_end, &yextent))
 	ylimit = ypos_end + yextent;
 
-      GtkStyleContext *sc = gtk_widget_get_style_context (w);
-
-      gtk_style_context_get (sc,
-			     GTK_STATE_FLAG_SELECTED,
-			     "background-color",
-			     &color, NULL);
-
-      GtkCssProvider *cp = gtk_css_provider_new ();
-      gchar *css = g_strdup_printf ("* {background-color: rgba(%d, %d, %d, 0.25);}",
-				    (int) (100.0 * color.red),
-				    (int) (100.0 * color.green),
-				    (int) (100.0 * color.blue));
-
-      gtk_css_provider_load_from_data (cp, css, strlen (css), 0);
-
-      gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (cp),
-				      GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
       gtk_render_background (sc, cr,
 			     xpos_start, ypos_start,
 			     xlimit - xpos_start,
 			     ylimit - ypos_start);
-
-      gtk_style_context_remove_provider (sc, GTK_STYLE_PROVIDER (cp));
-      g_free (css);
-      g_object_unref (cp);
     }
+
+ done:
+  gtk_style_context_remove_provider (sc, GTK_STYLE_PROVIDER (cp));
+  g_free (css);
+  g_object_unref (cp);
+  gdk_rgba_free (color);
 }
 
 gboolean
@@ -355,6 +361,21 @@ ssw_sheet_default_forward_conversion (SswSheet *sheet, GtkTreeModel *m,
 
 static GtkCellRenderer * choose_renderer (SswSheetBody *body, gint col, gint row);
 
+static const gchar *unfocused_border = "* {\n"
+  " border-width: 2px;\n"
+  " border-radius: 2px;\n"
+  " border-color: black;\n"
+  " border-style: dotted;\n"
+  "}";
+
+
+static const gchar *focused_border = "* {\n"
+  " border-width: 2px;\n"
+  " border-radius: 2px;\n"
+  " border-color: black;\n"
+  " border-style: solid;\n"
+  "}";
+
 static gboolean
 __draw (GtkWidget *widget, cairo_t *cr)
 {
@@ -372,6 +393,24 @@ __draw (GtkWidget *widget, cairo_t *cr)
 
   GtkStyleContext *sc = gtk_widget_get_style_context (widget);
 
+  GtkCssProvider *cp = gtk_css_provider_new ();
+
+  if ((gtk_widget_is_focus (widget) ||
+       (priv->editor && gtk_widget_is_focus (priv->editor))))
+    gtk_css_provider_load_from_data (cp, focused_border, strlen (focused_border), 0);
+  else
+    gtk_css_provider_load_from_data (cp, unfocused_border,
+				     strlen (unfocused_border), 0);
+
+  gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (cp),
+				  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+  GtkBorder border;
+  gtk_style_context_get_border (sc,
+				gtk_widget_get_state_flags (widget),
+				&border);
+
+
   int row = priv->vaxis->last_cell;
   gint y;
   for (y = priv->vaxis->cell_limits->len - 1;
@@ -409,51 +448,24 @@ __draw (GtkWidget *widget, cairo_t *cr)
 	  rect.height = vgeom->size;
 
 	  --col;
+	  if (col == active_col && row == active_row)
+	    {
+	      /* Draw frame */
+	      gtk_render_frame (sc, cr,
+				rect.x - border.left,
+				rect.y - border.top,
+				rect.width + border.left + border.right + 1,
+				rect.height + border.top + border.bottom + 1);
+	    }
 	  if (priv->data_model
 	      && row < gtk_tree_model_iter_n_children (priv->data_model, NULL)
 	      && col < gtk_tree_model_get_n_columns (priv->data_model))
 	    {
-	      GValue value = G_VALUE_INIT;
-
-	      gtk_tree_model_get_value (priv->data_model, &iter, col, &value);
-
 	      GtkCellRenderer *renderer = choose_renderer (body, col, row);
 
 	      if (GTK_IS_CELL_RENDERER_TEXT (renderer))
 		{
-		  const char *cell_text = NULL;
-
-		  if (col == active_col && row == active_row)
-		    {
-		      GtkCssProvider *cp = gtk_css_provider_new ();
-		      const gchar *css = "* {\n"
-			" border-width: 2px;\n"
-			" border-radius: 2px;\n"
-			" border-color: black;\n"
-			" border-style: solid;\n"
-			"}";
-
-		      gtk_css_provider_load_from_data (cp, css, strlen (css), 0);
-
-		      gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (cp),
-						      GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
-		      GtkBorder border;
-		      gtk_style_context_get_border (sc,
-						    gtk_widget_get_state_flags (widget),
-						    &border);
-
-		      /* Draw frame */
-		      gtk_render_frame (sc, cr,
-					rect.x - border.left,
-					rect.y - border.top,
-					rect.width + border.left + border.right + 1,
-					rect.height + border.top + border.bottom + 1);
-
-		      gtk_style_context_remove_provider (sc, GTK_STYLE_PROVIDER (cp));
-
-		      g_object_unref (cp);
-		    }
+		  char *cell_text = NULL;
 
 		  if (col != active_col || row != active_row ||
 		      (priv->sheet->selected_body != GTK_WIDGET (body)))
@@ -462,15 +474,20 @@ __draw (GtkWidget *widget, cairo_t *cr)
 			 It is already rendered by the cell_editable widget
 			 and rendering twice looks unaesthetic */
 
+		      GValue value = G_VALUE_INIT;
+		      gtk_tree_model_get_value (priv->data_model, &iter, col, &value);
 		      cell_text = priv->cf (priv->sheet, priv->data_model, col, row, &value);
+		      g_value_unset (&value);
 		    }
 
 		  g_object_set (renderer,
 				"text", cell_text,
 				NULL);
+
+		  g_free (cell_text);
 		}
 
-	      g_value_unset (&value);
+
 
 	      gtk_cell_renderer_render (renderer, cr, widget,
 					&rect, &rect, 0);
@@ -508,6 +525,10 @@ __draw (GtkWidget *widget, cairo_t *cr)
 
   draw_selection (body, cr);
 
+  gtk_style_context_remove_provider (sc, GTK_STYLE_PROVIDER (cp));
+
+  g_object_unref (cp);
+
   return GTK_WIDGET_CLASS (ssw_sheet_body_parent_class)->draw (widget, cr);
 }
 
@@ -531,7 +552,8 @@ __dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
-  g_object_unref (priv->data_model);
+  if (priv->data_model)
+    g_object_unref (priv->data_model);
 
   priv->dispose_has_run = TRUE;
 
@@ -594,7 +616,7 @@ set_selection (SswSheetBody *body, gint start_x, gint start_y, gint end_x, gint
    model.   Returns TRUE if COL or ROW was changed.
 */
 static gboolean
-trim_to_model_limits (SswSheetBody *body, gint *col, gint *row)
+trim_to_model_limits (SswSheetBody *body, gint old_col, gint old_row, gint *col, gint *row)
 {
   gboolean clipped = FALSE;
   PRIV_DECL (body);
@@ -602,6 +624,12 @@ trim_to_model_limits (SswSheetBody *body, gint *col, gint *row)
   /* Allow editing of a new cell */
   if (priv->editable)
     {
+      if (old_row == ssw_sheet_axis_get_size (priv->vaxis) && old_col == 0)
+	return FALSE;
+
+      if (old_col == ssw_sheet_axis_get_size (priv->haxis) && old_row == 0)
+	return FALSE;
+
       if (*row == ssw_sheet_axis_get_size (priv->vaxis) && *col == 0)
 	return FALSE;
 
@@ -663,6 +691,7 @@ static void
 start_editing (SswSheetBody *body, GdkEvent *e)
 {
   PRIV_DECL (body);
+
   gint row = -1, col = -1;
   get_active_cell (body, &col, &row);
 
@@ -700,19 +729,27 @@ __button_press_event (GtkWidget *w, GdkEventButton *e)
 
   GdkEventButton *eb = (GdkEventButton *) e;
 
+  gint old_row = -1, old_col = -1;
+  get_active_cell (body, &old_col, &old_row);
+
+
   gint col  = ssw_sheet_axis_find_cell (priv->haxis, eb->x, NULL, NULL);
   gint row  = ssw_sheet_axis_find_cell (priv->vaxis, eb->y, NULL, NULL);
 
-  if (trim_to_model_limits (body, &col, &row))
+  if (trim_to_model_limits (body, old_col, old_row, &col, &row))
     return FALSE;
 
-  set_active_cell (body, col, row);
-
   if (priv->sheet->selected_body != GTK_WIDGET (body))
     {
       destroy_editor_widget (SSW_SHEET_BODY (priv->sheet->selected_body));
     }
 
+  GdkWindow *win = gtk_widget_get_window (GTK_WIDGET (body));
+
+  if (gdk_window_get_cursor (win) != priv->hc
+      && gdk_window_get_cursor (win) != priv->vc)
+    set_active_cell (body, col, row);
+
   priv->sheet->selected_body = GTK_WIDGET (body);
   gtk_widget_grab_focus (w);
 
@@ -815,7 +852,7 @@ ssw_sheet_body_set_active_cell (SswSheetBody *body,
 
   start_editing (body, e);
 
-  /* Do not annouce the selection if it hasn't changed.
+  /* Do not announce the selection if it hasn't changed.
      Otherwise infinite loops can occur :( */
   if (oldrow != row || oldcol != col)
     {
@@ -834,7 +871,8 @@ __key_press_event (GtkWidget *w, GdkEventKey *e)
 
   if (is_printable_key (e->keyval))
     {
-      if (priv->editor && (priv->sheet->selected_body == GTK_WIDGET (body)))
+      if (priv->editor && priv->editable &&
+	  (priv->sheet->selected_body == GTK_WIDGET (body)))
 	{
 	  gtk_widget_grab_focus (priv->editor);
 	  if (GTK_IS_ENTRY (priv->editor))
@@ -939,11 +977,11 @@ __key_press_event (GtkWidget *w, GdkEventKey *e)
       }
       break;
     default:
-      return TRUE;
+      return FALSE;
       break;
     }
 
-  trim_to_model_limits (body, &col, &row);
+  trim_to_model_limits (body, old_col, old_row, &col, &row);
 
   if (col > ssw_sheet_axis_get_last (priv->haxis))
     ssw_sheet_axis_jump_end (priv->haxis, col);
@@ -1101,6 +1139,81 @@ select_entire_column (SswSheetBody *body, gint i, guint state, gpointer ud)
 }
 
 
+static void
+set_editor_widget_value (SswSheetBody *body, GValue *value, GtkEditable *editable)
+{
+  PRIV_DECL (body);
+
+  if (editable == NULL)
+    return;
+
+  gint row = -1, col = -1;
+  get_active_cell (body, &col, &row);
+
+  /* Allow the datum to be changed only if the "editable" property is set */
+  gtk_editable_set_editable (editable, priv->editable);
+  
+  if (GTK_IS_SPIN_BUTTON (editable))
+    {
+      if (G_IS_VALUE (value))
+	{
+	  GValue dvalue = G_VALUE_INIT;
+	  g_value_init (&dvalue, G_TYPE_DOUBLE);
+	  if (g_value_transform (value, &dvalue))
+	    {
+	      gtk_spin_button_set_value (GTK_SPIN_BUTTON (editable), g_value_get_double (&dvalue));
+	    }
+	  g_value_unset (&dvalue);
+	}
+    }
+  else if (GTK_IS_ENTRY (editable))
+    {
+      gchar *s = NULL;
+      if (G_IS_VALUE (value))
+	s = priv->cf (priv->sheet, priv->data_model, col, row, value);
+      gtk_entry_set_text (GTK_ENTRY (editable), s ? s : "");
+      g_free (s);
+    }
+  else if (GTK_IS_COMBO_BOX (editable))
+    {
+      gint n = -1;
+      if (G_IS_VALUE (value))
+	n = g_value_get_enum (value);
+      gtk_combo_box_set_active (GTK_COMBO_BOX (editable), n);
+    }
+  else
+    {
+      gchar *x = NULL;
+      if (G_IS_VALUE (value))
+	x = g_strdup_value_contents (value);
+      g_warning ("Unhandled edit widget %s, when dealing with %s\n",
+		 G_OBJECT_TYPE_NAME (editable), x);
+      g_free (x);
+    }
+}
+
+static void
+on_data_change (GtkTreeModel *tm, guint posn, guint rm, guint add, gpointer p)
+{
+  /* Set the "editor widget" (typically a GtkEntry) to reflect the new value */
+  SswSheetBody *body = SSW_SHEET_BODY (p);
+  PRIV_DECL (body);
+
+  gint row = -1, col = -1;
+  if (! get_active_cell (body, &col, &row))
+    return;
+
+  GtkTreeIter iter;
+  if (gtk_tree_model_iter_nth_child (priv->data_model, &iter, NULL, row))
+    {
+      GValue value = G_VALUE_INIT;
+      gtk_tree_model_get_value (priv->data_model, &iter, col, &value);
+      
+      set_editor_widget_value (body, &value, GTK_EDITABLE (priv->editor));
+      g_value_unset (&value);
+    }
+}
+
 static void
 __set_property (GObject *object,
                 guint prop_id, const GValue *value, GParamSpec *pspec)
@@ -1108,7 +1221,6 @@ __set_property (GObject *object,
   SswSheetBody *body = SSW_SHEET_BODY (object);
   PRIV_DECL (body);
 
-
   switch (prop_id)
     {
     case PROP_SHEET:
@@ -1140,6 +1252,7 @@ __set_property (GObject *object,
       break;
     case PROP_CONVERT_FWD_FUNC:
       priv->cf = g_value_get_pointer (value);
+      gtk_widget_queue_draw (GTK_WIDGET (body));
       break;
     case PROP_CONVERT_REV_FUNC:
       priv->revf = g_value_get_pointer (value);
@@ -1151,6 +1264,7 @@ __set_property (GObject *object,
       if (priv->data_model)
 	g_object_unref (priv->data_model);
       priv->data_model = g_value_get_object (value);
+      g_signal_connect (priv->data_model, "items-changed", G_CALLBACK (on_data_change), body);
       g_object_ref (priv->data_model);
       break;
     default:
@@ -1356,8 +1470,6 @@ selection_drag_begin (GtkGesture *gesture,
 
   GdkWindow *win = gtk_widget_get_window (GTK_WIDGET (body));
 
-  ssw_sheet_body_unset_selection (body);
-
   if (gdk_window_get_cursor (win) == priv->hc || gdk_window_get_cursor (win) == priv->vc)
     {
       gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
@@ -1366,6 +1478,8 @@ selection_drag_begin (GtkGesture *gesture,
 
   gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_CLAIMED);
 
+  ssw_sheet_body_unset_selection (body);
+
   priv->selection->start_x = ssw_sheet_axis_find_cell (priv->haxis, start_x, NULL, NULL);
   priv->selection->start_y = ssw_sheet_axis_find_cell (priv->vaxis, start_y, NULL, NULL);
 
@@ -2052,7 +2166,6 @@ on_editing_done (GtkCellEditable *e, gpointer ud)
   else if (GTK_IS_ENTRY (e))
     {
       const char *s = gtk_entry_get_text (GTK_ENTRY (e));
-
       priv->revf (priv->data_model, col, row, s, &value);
     }
   else if (GTK_IS_COMBO_BOX (e))
@@ -2121,6 +2234,10 @@ text_editing_started (GtkCellRenderer *cell,
   SswSheetBody *body = NULL;
   gint row = -1, col = -1;
   sscanf (path, "r%*dc%*ds%p", &body);
+
+  if (body == NULL) /* There is no active cell! */
+    return;
+
   get_active_cell (body, &col, &row);
   PRIV_DECL (body);
 
@@ -2174,8 +2291,12 @@ text_editing_started (GtkCellRenderer *cell,
 		"hconstraint", hsize,
 		"vconstraint", vsize,
 		NULL);
-  gtk_container_add (GTK_CONTAINER (priv->active_cell_holder), GTK_WIDGET (editable));
-  gtk_layout_move (GTK_LAYOUT (body), priv->active_cell_holder,  hlocation, vlocation);
+
+  gtk_container_add (GTK_CONTAINER (priv->active_cell_holder),
+		     GTK_WIDGET (editable));
+
+  gtk_layout_move (GTK_LAYOUT (body), priv->active_cell_holder,
+		   hlocation, vlocation);
 
   GtkTreeIter iter;
   GValue value = G_VALUE_INIT;
@@ -2199,52 +2320,24 @@ text_editing_started (GtkCellRenderer *cell,
   g_signal_connect (editable, "editing-done",
 		    G_CALLBACK (on_editing_done), body);
 
+
+  set_editor_widget_value (body, &value, GTK_EDITABLE (editable));
+  g_value_unset (&value);
+
   if (GTK_IS_SPIN_BUTTON (editable))
     {
-      if (G_IS_VALUE (&value))
-	{
-	  GValue dvalue = G_VALUE_INIT;
-	  g_value_init (&dvalue, G_TYPE_DOUBLE);
-	  if (g_value_transform (&value, &dvalue))
-	    {
-	      gtk_spin_button_set_value (GTK_SPIN_BUTTON (editable), g_value_get_double (&dvalue));
-	    }
-	  g_value_unset (&dvalue);
-	}
       g_signal_connect (editable, "value-changed", G_CALLBACK (finish_editing), NULL);
     }
   else if (GTK_IS_ENTRY (editable))
     {
       /* "activate" means when the Enter key is pressed */
-      g_signal_connect (editable, "activate",
-			G_CALLBACK (on_entry_activate), body);
-
-      gchar *s = NULL;
-      if (G_IS_VALUE (&value))
-	s = priv->cf (priv->sheet, priv->data_model, col, row, &value);
-      gtk_entry_set_text (GTK_ENTRY (editable), s ? s : "");
-      g_free (s);
+      g_signal_connect (editable, "activate", G_CALLBACK (on_entry_activate), body);
     }
   else if (GTK_IS_COMBO_BOX (editable))
     {
-      gint n = -1;
-      if (G_IS_VALUE (&value))
-	n = g_value_get_enum (&value);
-      gtk_combo_box_set_active (GTK_COMBO_BOX (editable), n);
       g_signal_connect_object (cell, "changed", G_CALLBACK (done_editing),
 			       editable, 0);
     }
-  else
-    {
-      gchar *x = NULL;
-      if (G_IS_VALUE (&value))
-	x = g_strdup_value_contents (&value);
-      g_warning ("Unhandled edit widget %s, when dealing with %s\n",
-		 G_OBJECT_TYPE_NAME (editable), x);
-      g_free (x);
-    }
-
-  g_value_unset (&value);
 
   gtk_widget_show_all (priv->active_cell_holder);
 }
