@@ -1,5 +1,5 @@
 /*
-  A candidate replacement for Pspp's sheet
+  A widget to display and manipulate tabular data
   Copyright (C) 2016, 2017  John Darrington
 
   This program is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "ssw-axis-model.h"
 #include "ssw-sheet-body.h"
 #include "ssw-marshaller.h"
+#include "ssw-xpaned.h"
 
 #define P_(X) (X)
 
@@ -170,8 +171,11 @@ __dispose (GObject *object)
   if (sheet->dispose_has_run)
     return;
 
-  g_object_unref (sheet->vmodel);
-  g_object_unref (sheet->hmodel);
+  if (sheet->vmodel)
+    g_object_unref (sheet->vmodel);
+
+  if (sheet->hmodel)
+    g_object_unref (sheet->hmodel);
 
   sheet->dispose_has_run = TRUE;
 
@@ -211,15 +215,15 @@ __set_property (GObject *object,
 	GType t = g_value_get_gtype (value);
 
 	if (t == GTK_TYPE_CONTAINER)
-	  t = GTK_TYPE_GRID;
+	  t = SSW_TYPE_XPANED;
 
-	GtkWidget *grid = g_object_new (t, NULL);
-
-	gtk_container_add (GTK_CONTAINER (sheet), grid);
+	GtkWidget *splitter = g_object_new (t, NULL);
+	gtk_container_add (GTK_CONTAINER (sheet), splitter);
 
 	for (i = 0; i < DIM * DIM ; ++i)
 	  {
-	    gtk_container_add_with_properties (GTK_CONTAINER (grid), sheet->sw[i],
+	    gtk_container_add_with_properties (GTK_CONTAINER (splitter),
+					       sheet->sw[i],
 					       "left-attach", i%DIM,
 					       "top-attach", i/DIM,
 					       NULL);
@@ -345,32 +349,29 @@ __set_property (GObject *object,
 	g_warning ("SswSheet can interpret list models only. Child nodes will be ignored.");
 
       if (sheet->vmodel == NULL)
-	{
-	  int n_rows = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (sheet->data_model), NULL);
-
-	  sheet->vmodel = g_object_new (SSW_TYPE_AXIS_MODEL,
-					"size", n_rows,
-					NULL);
-
-	  g_signal_connect (sheet->data_model, "items-changed",
-			    G_CALLBACK (resize_vmodel), sheet);
-
-	  arrange (sheet);
-	}
+	sheet->vmodel = g_object_new (SSW_TYPE_AXIS_MODEL, NULL);
 
       if (sheet->hmodel == NULL)
+	sheet->hmodel = g_object_new (SSW_TYPE_AXIS_MODEL,  NULL);
+
+      if (SSW_IS_AXIS_MODEL (sheet->vmodel))
+	{
+	  int n_rows = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (sheet->data_model), NULL);
+	  g_object_set (sheet->vmodel, "size", n_rows, NULL);
+	  g_signal_connect_object (sheet->data_model, "items-changed",
+				   G_CALLBACK (resize_vmodel), sheet, 0);
+	}
+
+      if (SSW_IS_AXIS_MODEL (sheet->hmodel))
 	{
 	  int n_cols = gtk_tree_model_get_n_columns (GTK_TREE_MODEL (sheet->data_model));
-
-	  sheet->hmodel = g_object_new (SSW_TYPE_AXIS_MODEL,
-					"size", n_cols,
-					NULL);
-
-	  g_signal_connect (sheet->data_model, "items-changed",
-			    G_CALLBACK (resize_hmodel), sheet);
-
-	  arrange (sheet);
+	  g_object_set (sheet->hmodel, "size", n_cols, NULL);
+	  g_signal_connect_object (sheet->data_model, "items-changed",
+				   G_CALLBACK (resize_hmodel), sheet, 0);
 	}
+
+      arrange (sheet);
+
       for (i = 0; i < DIM * DIM; ++i)
 	{
 	  g_object_set (sheet->sheet[i], "data-model", sheet->data_model, NULL);
@@ -482,7 +483,7 @@ ssw_sheet_class_init (SswSheetClass *class)
     g_param_spec_boolean ("editable",
 			  P_("Editable"),
 			  P_("True if the sheet is editable"),
-			  TRUE,
+			  FALSE,
 			  G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
   GParamSpec *renderer_func_spec =
@@ -735,6 +736,9 @@ on_drag_n_drop (SswSheet *sheet, gint from, gint to, GtkOrientable *axis)
 static void
 ssw_sheet_init (SswSheet *sheet)
 {
+  sheet->vmodel = g_object_new (SSW_TYPE_AXIS_MODEL, NULL);
+  sheet->hmodel = g_object_new (SSW_TYPE_AXIS_MODEL, NULL);
+
   gint i;
   for (i = 0; i < DIM; ++i)
     {
@@ -811,10 +815,10 @@ ssw_sheet_init (SswSheet *sheet)
     }
 
   sheet->renderer_func_datum = NULL;
-  sheet->vmodel = NULL;
-  sheet->hmodel = NULL;
   sheet->dispose_has_run = FALSE;
   sheet->selected_body = SSW_SHEET_SINGLE (sheet->sheet[0])->body;
+
+  arrange (sheet);
 }
 
 void
